/* Scrutari - Copyright (c) 2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets;

import fr.exemole.sctserver.tools.ErrorUtils;
import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logging.MessageLogBuilder;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
class ConfDOMReader {


    static Conf readConf(File f, MessageLogBuilder messageLogBuilder) {
        Document document = toDocument(f, messageLogBuilder);
        if (document == null) {
            return null;
        }
        Conf conf = new Conf();
        NodeList nodeList = document.getDocumentElement().getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++) {
            Node nd = nodeList.item(i);
            if (nd instanceof Element) {
                Element el = (Element) nd;
                String tagName = el.getTagName();
                String data = XMLUtils.getData(el);
                if ((tagName.equals("name")) || (tagName.equals("uuid"))) {
                    conf.name = data;
                } else if (tagName.equals("url")) {
                    if (data.length() > 0) {
                        if (!data.endsWith("/")) {
                            data = data + "/";
                        }
                    }
                    conf.url = data;
                } else if ((tagName.equals("default-lang")) || (tagName.equals("default-lang-ui"))) {
                    if (data.length() > 0) {
                        try {
                            conf.webappDefaultLang = Lang.parse(data);
                        } catch (ParseException pe) {
                            ErrorUtils.severe(messageLogBuilder, "_ error.wrong.xml.tagvalue", "/conf/" + tagName, data);
                        }
                    }
                }
            }
        }
        return conf;
    }

    static Dirs readDirs(String scrutariConfFileString, MessageLogBuilder messageLogBuilder) {
        Document document = toDocument(scrutariConfFileString, messageLogBuilder);
        if (document == null) {
            return null;
        }
        String confDirString = null;
        String varDirString = null;
        NodeList nodeList = document.getDocumentElement().getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++) {
            Node nd = nodeList.item(i);
            if (nd instanceof Element) {
                Element el = (Element) nd;
                String tagName = el.getTagName();
                String data = XMLUtils.getData(el);
                if (tagName.equals("conf-dir")) {
                    confDirString = data;
                } else if (tagName.equals("var-dir")) {
                    varDirString = data;
                }
            }
        }
        boolean critical = false;
        if ((confDirString == null) || (confDirString.length() == 0)) {
            ErrorUtils.critical(messageLogBuilder, "_ error.config.xml_missingtag", "/scrutari-conf/conf-dir");
            critical = true;
        }
        if ((varDirString == null) || (varDirString.length() == 0)) {
            ErrorUtils.critical(messageLogBuilder, "_ error.config.xml_missingtag", "/scrutari-conf/var-dir");
            critical = true;
        }
        if (critical) {
            return null;
        }
        return new Dirs(confDirString, varDirString);
    }


    private static Document toDocument(String scrutariConfFileString, MessageLogBuilder messageLogBuilder) {
        messageLogBuilder.setCurrentURI(scrutariConfFileString);
        File scrutariConfFile = new File(scrutariConfFileString);
        if (!scrutariConfFile.exists()) {
            ErrorUtils.critical(messageLogBuilder, "_ error.config.conffile_missing", scrutariConfFileString);
            return null;
        }
        if (scrutariConfFile.isDirectory()) {
            ErrorUtils.critical(messageLogBuilder, "_ error.config.conffile_dir", scrutariConfFileString);
            return null;
        }
        return toDocument(scrutariConfFile, messageLogBuilder);
    }

    static Document toDocument(File f, MessageLogBuilder messageLogBuilder) {
        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document doc = db.parse(f);
            return doc;
        } catch (ParserConfigurationException pce) {
            ErrorUtils.critical(messageLogBuilder, "_ error.config.parseconfig", pce.getMessage());
            return null;
        } catch (IOException ioe) {
            ErrorUtils.critical(messageLogBuilder, "_ error.config.conffile_io", f.getPath(), ioe.getMessage());
            return null;
        } catch (SAXException sax) {
            ErrorUtils.critical(messageLogBuilder, "_ error.config.conffile_xml", f.getPath(), sax.getMessage());
            return null;
        }
    }


    static class Dirs {

        private final String confDirString;
        private final String varDirString;

        private Dirs(String confDirString, String varDirString) {
            this.confDirString = confDirString;
            this.varDirString = varDirString;
        }

        String getConfDirString() {
            return confDirString;
        }

        String getVarDirString() {
            return varDirString;
        }


    }


    static class Conf {

        private String name = "scrutari.localhost";
        private String url = "http://localhost:8080/scrutari/";
        private Lang webappDefaultLang = ScrutariWebapp.DEFAULT_LANG;

        public String getName() {
            return name;
        }

        public String getUrl() {
            return url;
        }

        public Lang getWebappDefaultLang() {
            return webappDefaultLang;
        }

    }

}
