/* ScrutariLib_DB - Copyright (c) 2012-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.db.tools;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.attr.AttributeDef;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.attr.AttributeUtils;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.lexie.AlineaInteger;


/**
 *
 * @author Vincent Calame
 */
public class AttributeKeyCoderBuilder {

    private final Map<AttributeKey, AttributeDef> existingMap = new HashMap<AttributeKey, AttributeDef>();
    private final List<AttributeDef> primaryList = new ArrayList<AttributeDef>();
    private final List<AttributeDef> secondaryList = new ArrayList<AttributeDef>();
    private final List< AttributeDef> technicalList = new ArrayList<AttributeDef>();

    public AttributeKeyCoderBuilder() {
    }


    public void addAttributeDef(String groupName, AttributeDef attributeDef) {
        List<AttributeDef> list = getList(groupName);
        if (list == null) {
            throw new IllegalArgumentException("Unknown groupName: " + groupName);
        }
        if (!existingMap.containsKey(attributeDef.getAttributeKey())) {
            if (list.size() == 999) {
                return;
            }
            list.add(attributeDef);
            existingMap.put(attributeDef.getAttributeKey(), attributeDef);
        }
    }

    public void addAttributeDefArray(String groupName, AttributeDef[] attributeDefArray) {
        List<AttributeDef> list = getList(groupName);
        if (list == null) {
            throw new IllegalArgumentException("Unknown groupName: " + groupName);
        }
        for (AttributeDef attributeDef : attributeDefArray) {
            if (!existingMap.containsKey(attributeDef.getAttributeKey())) {
                if (list.size() == 999) {
                    break;
                }
                list.add(attributeDef);
                existingMap.put(attributeDef.getAttributeKey(), attributeDef);
            }
        }
    }

    public AttributeKeyCoder toAttributeKeyCoder() {
        Map<String, List<AttributeDef>> attributeDefListMap = new HashMap<String, List<AttributeDef>>();
        attributeDefListMap.put(AttributeKeyCoder.PRIMARY_GROUP, AttributeUtils.toImmutableList(primaryList));
        attributeDefListMap.put(AttributeKeyCoder.SECONDARY_GROUP, AttributeUtils.toImmutableList(secondaryList));
        attributeDefListMap.put(AttributeKeyCoder.TECHNICAL_GROUP, AttributeUtils.toImmutableList(technicalList));
        Map<AttributeKey, AttributeDef> attributeDefMap = new HashMap<AttributeKey, AttributeDef>();
        attributeDefMap.putAll(existingMap);
        Map<AttributeKey, Integer> alineaRootCodeMap = new HashMap<AttributeKey, Integer>();
        int primaryMax = AlineaInteger.PRIMARY_ATTRIBUTEGROUP_START;
        for (AttributeDef attributeDef : primaryList) {
            alineaRootCodeMap.put(attributeDef.getAttributeKey(), primaryMax);
            primaryMax++;
        }
        int secondaryMax = AlineaInteger.SECONDARY_ATTRIBUTEGROUP_START;
        for (AttributeDef attributeDef : secondaryList) {
            alineaRootCodeMap.put(attributeDef.getAttributeKey(), secondaryMax);
            secondaryMax++;
        }
        Integer itg = AttributeKeyCoder.TECHNICAL_ATTRIBUTE;
        for (AttributeDef attributeDef : technicalList) {
            alineaRootCodeMap.put(attributeDef.getAttributeKey(), itg);
        }
        return new InternalAttributeKeyCoder(attributeDefListMap, attributeDefMap, alineaRootCodeMap);
    }

    private List<AttributeDef> getList(String name) {
        if (name.equals(AttributeKeyCoder.PRIMARY_GROUP)) {
            return primaryList;
        }
        if (name.equals(AttributeKeyCoder.SECONDARY_GROUP)) {
            return secondaryList;
        }
        if (name.equals(AttributeKeyCoder.TECHNICAL_GROUP)) {
            return technicalList;
        }
        return null;
    }

    private void addAttributeDefArray(Map<String, AttributeDef[]> map, String groupName) {
        AttributeDef[] array = map.get(groupName);
        if (array != null) {
            addAttributeDefArray(groupName, array);
        }
    }

    public static AttributeKeyCoder build(Map<String, AttributeDef[]> map) {
        AttributeKeyCoderBuilder builder = new AttributeKeyCoderBuilder();
        if (map != null) {
            builder.addAttributeDefArray(map, AttributeKeyCoder.PRIMARY_GROUP);
            builder.addAttributeDefArray(map, AttributeKeyCoder.SECONDARY_GROUP);
            builder.addAttributeDefArray(map, AttributeKeyCoder.TECHNICAL_GROUP);
        }
        return builder.toAttributeKeyCoder();
    }


    private static class InternalAttributeKeyCoder implements AttributeKeyCoder {

        private final Map<String, List<AttributeDef>> attributeDefListMap;
        private final Map<AttributeKey, AttributeDef> attributeDefMap;
        private final Map<AttributeKey, Integer> alineaRootCodeMap;

        private InternalAttributeKeyCoder(Map<String, List<AttributeDef>> attributeDefListMap, Map<AttributeKey, AttributeDef> attributeDefMap, Map<AttributeKey, Integer> alineaRootCodeMap) {
            this.attributeDefListMap = attributeDefListMap;
            this.attributeDefMap = attributeDefMap;
            this.alineaRootCodeMap = alineaRootCodeMap;
        }

        @Override
        public int getAttributeDefCount() {
            return attributeDefMap.size();
        }

        @Override
        public int getAlineaRootCode(AttributeKey attributeKey) {
            Integer itg = alineaRootCodeMap.get(attributeKey);
            if (itg == null) {
                return AttributeKeyCoder.UNKNOWN_ATTRIBUTE;
            }
            return itg;
        }

        @Override
        public List<AttributeDef> getAttributeDefList(String name) {
            List<AttributeDef> list = attributeDefListMap.get(name);
            if (list == null) {
                return AttributeUtils.EMPTY_ATTRIBUTEDEFLIST2;
            }
            return list;
        }

        @Override
        public AttributeDef getAttributeDef(AttributeKey attributeKey) {
            return attributeDefMap.get(attributeKey);
        }

    }

}
