/* ScrutariLib_SearchEngine - Copyright (c) 2015-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.options;

import java.util.HashSet;
import java.util.Set;
import net.mapeadores.util.localisation.Lang;
import net.scrutari.searchengine.api.options.ComputedOptions;
import net.scrutari.searchengine.api.options.PertinencePonderation;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.tools.EligibilityBuffer;


/**
 *
 * @author Vincent Calame
 */
public class ComputedOptionsBuilder {

    private final SearchOptions searchOptions;
    private boolean mandatoryOperand = false;
    private boolean allLang = false;
    private Lang[] langArray = null;
    private int[] codecorpusArray = null;
    private int[] codethesaurusArray = null;
    private Set<Integer> codeficheSet = null;
    private EligibilityBuffer eligibilityBuffer = null;
    private int ficheMaximum = 0;

    public ComputedOptionsBuilder(SearchOptions searchOptions) {
        this.searchOptions = searchOptions;
    }

    public void setMandatoryOperand(boolean mandatoryOperand) {
        this.mandatoryOperand = mandatoryOperand;
    }

    public void setEligibilityBuffer(EligibilityBuffer eligibilityBuffer) {
        this.eligibilityBuffer = eligibilityBuffer;
    }

    public void setCodecorpusArray(int[] codecorpusArray) {
        this.codecorpusArray = codecorpusArray;
    }

    public void setCodethesaurusArray(int[] codethesaurusArray) {
        this.codethesaurusArray = codethesaurusArray;
    }

    public void setLangArray(boolean allLang, Lang[] langArray) {
        this.allLang = allLang;
        this.langArray = langArray;
    }

    public void addCodefiche(int codefiche) {
        if (codeficheSet == null) {
            codeficheSet = new HashSet<Integer>();
        }
        codeficheSet.add(codefiche);
    }

    public void setFicheMaximum(int ficheMaximum) {
        this.ficheMaximum = ficheMaximum;
    }

    public ComputedOptions toComputedOptions() {
        return new InternalComputedOptions(searchOptions, mandatoryOperand, allLang, langArray, codecorpusArray, codethesaurusArray, codeficheSet, eligibilityBuffer, ficheMaximum);
    }


    private static class InternalComputedOptions implements ComputedOptions {

        private final SearchOptions searchOptions;
        private final boolean mandatoryOperand;
        private final boolean allLang;
        private final Lang[] langArray;
        private final int[] codecorpusArray;
        private final int[] codethesaurusArray;
        private final Set<Integer> codeficheSet;
        private final EligibilityBuffer eligibilityBuffer;
        private final int ficheMaximum;

        private InternalComputedOptions(SearchOptions searchOptions, boolean mandatoryOperand, boolean allLang, Lang[] langArray, int[] codecorpusArray, int[] codethesaurusArray, Set<Integer> codeficheSet, EligibilityBuffer eligibilityBuffer, int ficheMaximum) {
            this.mandatoryOperand = mandatoryOperand;
            this.searchOptions = searchOptions;
            this.allLang = allLang;
            this.langArray = langArray;
            this.codecorpusArray = codecorpusArray;
            this.codethesaurusArray = codethesaurusArray;
            this.codeficheSet = codeficheSet;
            this.eligibilityBuffer = eligibilityBuffer;
            this.ficheMaximum = ficheMaximum;
        }

        @Override
        public SearchOptions getSearchOptions() {
            return searchOptions;
        }

        @Override
        public PertinencePonderation getPertinencePonderation() {
            if (searchOptions == null) {
                return null;
            } else {
                return searchOptions.getSearchOptionsDef().getPertinencePonderation();
            }
        }

        @Override
        public boolean hasMandatoryOperand() {
            return mandatoryOperand;
        }

        @Override
        public Lang[] getLangArray() {
            return langArray;
        }

        @Override
        public boolean isAllLang() {
            return allLang;
        }

        @Override
        public int[] getCodecorpusArray() {
            return codecorpusArray;
        }

        @Override
        public int[] getCodethesaurusArray() {
            return codethesaurusArray;
        }

        @Override
        public boolean acceptFiche(int codefiche) {
            if (codeficheSet != null) {
                if (!codeficheSet.contains(codefiche)) {
                    return false;
                }
            }
            if (eligibilityBuffer == null) {
                return true;
            }
            return eligibilityBuffer.acceptFiche(codefiche);
        }

        @Override
        public boolean acceptMotcle(int codemotcle) {
            if (eligibilityBuffer == null) {
                return true;
            }
            return eligibilityBuffer.acceptMotcle(codemotcle);
        }

        @Override
        public int getFicheMaximum() {
            return ficheMaximum;
        }

    }

}
