/* ScrutariLib_Xml - Copyright (c) 2011-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.xml.xmlpull;

import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.text.ParseException;
import java.util.Map;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.xml.xmlpull.LabelElementHandler;
import net.mapeadores.util.xml.xmlpull.XmlPullElementHandler;
import net.scrutari.data.ScrutariAttributes;
import net.scrutari.xml.ScrutariXmlConstants;


/**
 *
 * @author Vincent Calame
 */
public final class XmlPullUtils {

    private XmlPullUtils() {
    }

    public static Label testLabel(XmlPullElementHandler elementHandler) {
        if (elementHandler instanceof LabelElementHandler) {
            return ((LabelElementHandler) elementHandler).toLabel();
        } else {
            return null;
        }
    }

    public static URI testURI(String href) throws MalformedURLException, URISyntaxException {
        if ((href == null) || (href.length() == 0)) {
            return null;
        }
        URI uri = new URI(href);
        uri.toURL();
        return uri;
    }

    public static AttributeKey parseAttributeKey(Map<String, String> attributeMap, MessageHandler messageHandler, String xpath) {
        String localKey = attributeMap.get("key");
        if (localKey != null) {
            localKey = localKey.trim();
        }
        if ((localKey == null) || (localKey.isEmpty())) {
            warning(messageHandler, "_ error.empty.xml.attribute", xpath, "key");
            return null;
        }
        String nameSpace = attributeMap.get("ns");
        if (nameSpace != null) {
            nameSpace = nameSpace.trim();
        }
        if ((nameSpace == null) || (nameSpace.isEmpty())) {
            warning(messageHandler, "_ error.empty.xml.attribute", xpath + "[@key='" + localKey + "']", "ns");
            return null;
        }
        try {
            AttributeKey attributeKey = ScrutariAttributes.toAttributeKey(nameSpace, localKey);
            return attributeKey;
        } catch (ParseException pe) {
            warning(messageHandler, "_ error.wrong.xml.attributevalue", xpath + "[@key='" + localKey + "' AND ns='" + nameSpace + "']", "key|ns", nameSpace + ":" + localKey);
            return null;
        }
    }

    public static void warning(MessageHandler messageHandler, String key, Object... values) {
        messageHandler.addMessage(ScrutariXmlConstants.WARNING_CATEGORY, LocalisationUtils.toMessage(key, values));
    }

    public static void warning(MessageHandler messageHandler, Message message) {
        messageHandler.addMessage(ScrutariXmlConstants.WARNING_CATEGORY, message);
    }

}
