/* SctServer - Copyright (c) 2012-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.json.MotcleDataJson;
import fr.exemole.sctserver.request.MotcleFieldsFilter;
import java.io.IOException;
import net.mapeadores.util.json.JSONWriter;
import net.scrutari.data.MotcleData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.db.api.codes.Codes;


/**
 *
 * @author Vincent Calame
 */
public class MotcleJsonProducer extends AbstractJsonProducer {

    private final int version;
    private final ScrutariDB scrutariDB;
    private final Codes motcleCodes;
    private final MotcleFieldsFilter motcleFieldsFilter;

    public MotcleJsonProducer(int version, ScrutariDB scrutariDB, Codes motcleCodes, MotcleFieldsFilter motcleFieldsFilter) {
        this.version = version;
        this.scrutariDB = scrutariDB;
        this.motcleCodes = motcleCodes;
        this.motcleFieldsFilter = motcleFieldsFilter;
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        jsonWriter.object();
        if (version > 1) {
            printVersion2(jsonWriter, dataAccess);
        } else {
            printVersion1(jsonWriter, dataAccess);
        }
        writeWarnings(jsonWriter);
        jsonWriter.endObject();
        dataAccess.close();
    }

    private void printVersion2(JSONWriter jsonWriter, DataAccess dataAccess) throws IOException {
        int motcleCount = motcleCodes.getCodeCount();
        jsonWriter.key("motcleArray");
        jsonWriter.array();
        for (int i = 0; i < motcleCount; i++) {
            int codemotcle = motcleCodes.getCode(i);
            MotcleData motcleData = dataAccess.getMotcleDataByCode(codemotcle);
            jsonWriter.object();
            MotcleDataJson.object(jsonWriter, motcleData, motcleFieldsFilter, null, null);
            jsonWriter.endObject();
        }
        jsonWriter.endArray();
    }

    private void printVersion1(JSONWriter jsonWriter, DataAccess dataAccess) throws IOException {
        jsonWriter.key("motcleData");
        jsonWriter.object();
        int motcleCount = motcleCodes.getCodeCount();
        jsonWriter.key("motcleCount");
        jsonWriter.value(motcleCount);
        if (motcleCount > 0) {
            jsonWriter.key("motcleArray");
            jsonWriter.array();
            for (int i = 0; i < motcleCount; i++) {
                int codemotcle = motcleCodes.getCode(i);
                MotcleData motcleData = dataAccess.getMotcleDataByCode(codemotcle);
                jsonWriter.object();
                MotcleDataJson.object(jsonWriter, motcleData, motcleFieldsFilter, null, null);
                jsonWriter.endObject();
            }
            jsonWriter.endArray();
        }
        jsonWriter.endObject();
    }

}
