/* SctServer - Copyright (c) 2006-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.dom;

import fr.exemole.sctserver.tools.ErrorUtils;
import java.text.ParseException;
import net.mapeadores.util.attr.AppendableAttributeHolderBuilder;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.xml.XMLUtils;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.URIParseException;
import net.scrutari.datauri.URIParser;
import net.scrutari.searchengine.api.options.CategoryDef;
import net.scrutari.searchengine.tools.options.CategoryDefBuilder;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class CategoriesDOMReader {

    private final Builder builder;
    private final MessageHandler messageHandler;

    public CategoriesDOMReader(Builder builder, MessageHandler messageHandler) {
        this.builder = builder;
        this.messageHandler = messageHandler;
    }

    public void readCategories(Element element_xml, String xpath) {
        xpath = xpath + "/" + element_xml.getTagName();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("category")) {
                    addCategory(element, xpath);
                }
            }
        }
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("corpus-list")) {
                    addCorpusList(element, xpath);
                } else if (!tagName.equals("category")) {
                    ErrorUtils.warning(messageHandler, "_ error.unknown.xml.tag", xpath + "/" + tagName);
                }
            }
        }
    }

    private void addCategory(Element element_xml, String xpath) {
        xpath = xpath + "/" + element_xml.getTagName();
        String name = element_xml.getAttribute("name");
        if (name.length() == 0) {
            ErrorUtils.severe(messageHandler, "_ error.empty.xml.attribute", xpath, "name");
            return;
        }
        xpath = xpath + "[@name='" + name + "']";
        CategoryDefBuilder categoryDefBuilder = builder.addCategory(name);
        AppendableAttributeHolderBuilder appendableAttributeHolderBuilder = new AppendableAttributeHolderBuilder();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if ((tagName.equals("label")) || (tagName.equals("lib"))) {
                    try {
                        LabelUtils.readLabel(element, categoryDefBuilder);
                    } catch (ParseException lie) {
                        ErrorUtils.warning(messageHandler, "_ error.wrong.xml.attributevalue", xpath + "/" + tagName, "xml:lang");
                    }
                } else if (tagName.equals("attr")) {
                    AttributeUtils.readAttrElement(appendableAttributeHolderBuilder, element);
                } else if (tagName.equals("phrase")) {
                    ConfDOMUtils.readPhraseElement(categoryDefBuilder.getPhrasesBuilder(), element, messageHandler, xpath + "/" + tagName);
                } else {
                    ErrorUtils.warning(messageHandler, "_ error.unknown.xml.tag", xpath + "/" + tagName);
                }
            }
        }
        categoryDefBuilder.addAttributes(appendableAttributeHolderBuilder.toAttributeHolder());
    }

    private void addCorpusList(Element element_xml, String xpath) {
        xpath = xpath + "/" + element_xml.getTagName();
        String name = element_xml.getAttribute("category-name");
        if (name.length() == 0) {
            ErrorUtils.severe(messageHandler, "_ error.empty.xml.attribute", xpath, "category-name");
            return;
        }
        xpath = xpath + "[@category-name='" + name + "']";
        if (name.equals(CategoryDef.DEFAULT_NAME)) {
            ErrorUtils.warning(messageHandler, "_ error.wrong.xml.attributevalue_withreason", xpath, "category-name", name, CategoryDef.DEFAULT_NAME + " cannot be use to define corpus list");
            return;
        }
        CategoryDefBuilder categoryDefBuilder = builder.getCategory(name);
        if (categoryDefBuilder == null) {
            ErrorUtils.severe(messageHandler, "_ error.unknown.xml.attributevalue", xpath, "category-name", name);
            return;
        }
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("uri")) {
                    String uriString = XMLUtils.getData(element);
                    if (uriString.length() == 0) {
                        ErrorUtils.severe(messageHandler, "_ error.empty.xml.tag", xpath + "/" + tagName);
                        continue;
                    }
                    try {
                        ScrutariDataURI scrutariDataURI = URIParser.parse(uriString);
                        if (scrutariDataURI instanceof CorpusURI) {
                            boolean done = builder.addCorpusURI(categoryDefBuilder, (CorpusURI) scrutariDataURI);
                            if (!done) {
                                ErrorUtils.warning(messageHandler, "_ error.existing.xml.tagvalue", xpath + "/" + tagName, uriString);
                            }
                        }
                    } catch (URIParseException uripe) {
                        ErrorUtils.severe(messageHandler, "_ error.wrong.xml.tagvalue_uri", xpath + "/" + tagName, uriString, uripe.getMessage());
                    }
                } else {
                    ErrorUtils.warning(messageHandler, "_ error.unknown.xml.tag", xpath + "/" + tagName);
                }
            }
        }
    }


    public static interface Builder {

        public CategoryDefBuilder addCategory(String name);

        public CategoryDefBuilder getCategory(String name);

        public boolean addCorpusURI(CategoryDefBuilder builder, CorpusURI corpusURI);

    }

}
