/* SctServer - Copyright (c) 2009-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.dom;

import fr.exemole.sctserver.tools.ErrorUtils;
import net.mapeadores.util.conditions.ConditionsUtils;
import net.mapeadores.util.conditions.TextTest;
import net.mapeadores.util.logging.MessageHandler;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class SearchLogFilterDOMReader {

    private final MessageHandler messageHandler;
    private final Builder builder;

    public SearchLogFilterDOMReader(Builder builder, MessageHandler messageHandler) {
        this.messageHandler = messageHandler;
        this.builder = builder;
    }

    public void readSearchLogFilter(Element element_xml, String xpath) {
        xpath = xpath + "/" + element_xml.getTagName();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("deny")) {
                    addDeny(element, xpath);
                } else {
                    ErrorUtils.warning(messageHandler, "_ error.unknown.xml.tag", xpath + "/" + tagName);
                }
            }
        }
    }

    private void addDeny(Element element_xml, String xpath) {
        xpath = xpath + "/" + element_xml.getTagName();
        TextTest ipCondition = ConditionsUtils.parseTextTest(element_xml.getAttribute("ip"));
        TextTest siteCondition = ConditionsUtils.parseTextTest(element_xml.getAttribute("site"));
        TextTest refCondition = ConditionsUtils.parseTextTest(element_xml.getAttribute("ref"));
        if ((ipCondition == null) && (refCondition == null) && (siteCondition == null)) {
            ErrorUtils.severe(messageHandler, "_ error.empty.xml.attribute", xpath, "deny");
            return;
        }
        builder.addDeny(ipCondition, siteCondition, refCondition);
    }


    public static interface Builder {

        public void addDeny(TextTest ipCondition, TextTest siteCondition, TextTest refCondition);

    }

}
