/* SctServer - Copyright (c) 2016-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.factory;

import fr.exemole.sctserver.api.EngineMetadata;
import fr.exemole.sctserver.tools.ErrorUtils;
import fr.exemole.sctserver.tools.conf.dom.MetadataDOMReader;
import java.io.File;
import java.io.IOException;
import java.util.List;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.util.logging.CategorizedMessageEntry;
import net.mapeadores.util.logging.SimpleMessageHandler;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public class MetadataConf {

    private String processURI;
    private List<CategorizedMessageEntry> messageList;
    private EngineMetadata engineMetadata;

    private MetadataConf() {

    }

    String getProcessURI() {
        return processURI;
    }

    List<CategorizedMessageEntry> getMessageList() {
        return messageList;
    }

    EngineMetadata getEngineMetadata() {
        return engineMetadata;
    }

    static MetadataConf load(File f) throws IOException, ParserConfigurationException {
        MetadataConf conf = new MetadataConf();
        conf.init(f);
        return conf;
    }

    private void init(File f) throws IOException, ParserConfigurationException {
        processURI = f.getPath();
        SimpleMessageHandler simpleHandler = new SimpleMessageHandler();
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        Document document;
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            document = db.parse(f);
        } catch (SAXException saxe) {
            ErrorUtils.xml(simpleHandler, "_ error.exception.xml.sax", saxe.getMessage());
            messageList = simpleHandler.toList();
            return;
        }
        MetadataDOMReader metadataDOMReader = new MetadataDOMReader(simpleHandler);
        engineMetadata = metadataDOMReader.readMetadata(document.getDocumentElement(), "");
        messageList = simpleHandler.toList();
    }

}
