/* SctServer - Copyright (c) 2016-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.context.dom;

import fr.exemole.sctserver.tools.ErrorUtils;
import fr.exemole.sctserver.tools.conf.dom.ConfDOMUtils;
import fr.exemole.sctserver.tools.context.EngineGroupBuilder;
import java.text.ParseException;
import net.mapeadores.util.attr.AppendableAttributeHolderBuilder;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class EngineGroupDOMReader {

    private final EngineGroupBuilder engineGroupBuilder;
    private final MessageHandler messageHandler;

    public EngineGroupDOMReader(EngineGroupBuilder engineGroupBuilder, MessageHandler messageHandler) {
        this.engineGroupBuilder = engineGroupBuilder;
        this.messageHandler = messageHandler;
    }

    public void readEngineGroup(Element element, String xpath) {
        RootHandler rootHandler = new RootHandler(xpath);
        DOMUtils.readChildren(element, rootHandler);
        rootHandler.flush();
    }


    private class RootHandler implements ElementHandler {

        private final String rootXpath;
        private final AppendableAttributeHolderBuilder appendableAttributeHolderBuilder = new AppendableAttributeHolderBuilder();

        private RootHandler(String xpath) {
            this.rootXpath = xpath;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            String xpath = rootXpath + "/" + tagName;
            if (tagName.equals("engine")) {
                String engineName = element.getAttribute("name").trim();
                if (engineName.length() == 0) {
                    ErrorUtils.severe(messageHandler, "_ error.empty.xml.attribute", xpath, "name");
                } else if (StringUtils.isTechnicalName(engineName, true)) {
                    engineGroupBuilder.addEngine(engineName);
                } else {
                    ErrorUtils.severe(messageHandler, "_ error.wrong.xml.attributevalue", xpath, "name", engineName);
                }
            } else if (tagName.equals("label")) {
                try {
                    LabelUtils.readLabel(element, engineGroupBuilder);
                } catch (ParseException ile) {
                    ErrorUtils.severe(messageHandler, "_ error.wrong.xml.langattributevalue", xpath, element.getAttribute("xml:lang"));
                }
            } else if (tagName.equals("attr")) {
                AttributeUtils.readAttrElement(appendableAttributeHolderBuilder, element);
            } else if (tagName.equals("phrase")) {
                ConfDOMUtils.readPhraseElement(engineGroupBuilder.getPhrasesBuilder(), element, messageHandler, xpath);
            } else {
                ErrorUtils.warning(messageHandler, "_ error.unknown.xml.tag", xpath);
            }
        }

        private void flush() {
            engineGroupBuilder.addAttributes(appendableAttributeHolderBuilder.toAttributeHolder());
        }

    }

}
