/* UtilLib - Copyright (c) 2011-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.html;

import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.LogUtils;


/**
 *
 * @author Vincent Calame
 */
public final class HtmlPrinterUtils {

    private HtmlPrinterUtils() {
    }

    public static void printCommandMessage(HtmlPrinter hp, CommandMessage commandMessage, HtmlAttributes doneAttributes, HtmlAttributes errorAttributes) {
        HtmlAttributes spanAttr = doneAttributes;
        if (commandMessage.isErrorMessage()) {
            spanAttr = errorAttributes;
        }
        hp
                .P()
                .SPAN(spanAttr)
                .localize(commandMessage)
                ._SPAN();
        if (commandMessage.hasMultiError()) {
            hp
                    .SPAN(errorAttributes)
                    .SMALL();
            for (Message message : commandMessage.getMultiErrorList()) {
                hp
                        .BR()
                        .localize(message);
            }
            hp
                    ._SMALL()
                    ._SPAN();
        } else if (commandMessage.hasLog()) {
            hp
                    .BR()
                    .PRE()
                    .escape(LogUtils.toString(commandMessage.getMessageLog()), true)
                    ._PRE();
        }
        hp
                ._P();
    }

    public static HtmlAttributes newFormAttributes(CommandBoxParameters commandBoxParameters) {
        if (commandBoxParameters == null) {
            return new HtmlAttributes();
        }
        HtmlAttributes attr = commandBoxParameters.getFormAttributes();
        if (attr != null) {
            return attr.copy();
        } else {
            return new HtmlAttributes();
        }
    }

    public static HtmlAttributes newFieldsetAttributes(CommandBoxParameters commandBoxParameters) {
        if (commandBoxParameters == null) {
            return new HtmlAttributes();
        }
        HtmlAttributes attr = commandBoxParameters.getFieldsetAttributes();
        if (attr != null) {
            return attr.copy();
        } else {
            return new HtmlAttributes();
        }
    }

    public static HtmlAttributes newLegendAttributes(CommandBoxParameters commandBoxParameters) {
        if (commandBoxParameters == null) {
            return new HtmlAttributes();
        }
        HtmlAttributes attr = commandBoxParameters.getLegendAttributes();
        if (attr != null) {
            return attr.copy();
        } else {
            return new HtmlAttributes();
        }
    }

    public static HtmlAttributes newCommandKeyDivAttributes(CommandBoxParameters commandBoxParameters) {
        if (commandBoxParameters == null) {
            return new HtmlAttributes();
        }
        HtmlAttributes attr = commandBoxParameters.getCommandKeyDivAttributes();
        if (attr != null) {
            return attr.copy();
        } else {
            return new HtmlAttributes();
        }
    }

    public static HtmlAttributes newSubmitDivAttributes(CommandBoxParameters commandBoxParameters) {
        if (commandBoxParameters == null) {
            return new HtmlAttributes();
        }
        HtmlAttributes attr = commandBoxParameters.getSubmitDivAttributes();
        if (attr != null) {
            return attr.copy();
        } else {
            return new HtmlAttributes();
        }
    }

}
