/* UtilLib - Copyright (c) 2013-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text;

import java.io.Serializable;


/**
 *
 * @author Vincent Calame
 */
public final class FileName implements Serializable {

    private static final long serialVersionUID = 1L;
    private final String basename;
    private final String extension;

    private FileName(String basename, String extension) {
        this.basename = basename;
        this.extension = extension;
    }

    /**
     * Jamais nul ou de longueur nulle
     */
    public String getBasename() {
        return basename;
    }

    /**
     * Jamais nul ou de longueur nulle
     */
    public String getExtension() {
        return extension;
    }

    @Override
    public String toString() {
        return basename + "." + extension;
    }

    public static FileName parse(String s) {
        if (s == null) {
            return null;
        }
        int length = s.length();
        if (length < 3) {
            return null;
        }
        int idx = s.lastIndexOf('.');
        if (idx < 1) {
            return null;
        }
        if (idx == (length - 1)) {
            return null;
        }
        return new FileName(s.substring(0, idx), s.substring(idx + 1));
    }

    public static FileName parse(String s, String defaultExtension) {
        if (s == null) {
            return null;
        }
        int length = s.length();
        if (length == 0) {
            return null;
        }
        int idx = s.lastIndexOf('.');
        if (idx == -1) {
            return new FileName(s, defaultExtension);
        } else if (idx == 0) {
            if (length == 1) {
                return null;
            }
            return new FileName(s.substring(1), defaultExtension);
        } else if (idx == (length - 1)) {
            return new FileName(s.substring(0, length - 1), defaultExtension);
        } else {
            return new FileName(s.substring(0, idx), s.substring(idx + 1));
        }
    }

}
